@@ -1,3 +1,4 @@
+#' @importFrom stats is.empty.model model.frame model.matrix model.offset na.omit na.pass reformulate relevel terms
 Diff <- function(player1, player2, formula = NULL, id = "..", data = NULL,
                  separate.ability = NULL, refcat = NULL, contrasts = NULL,
                  subset = NULL) {
@@ -8,6 +9,10 @@ Diff <- function(player1, player2, formula = NULL, id = "..", data = NULL,
         !identical(levels(player.one), levels(player.two)))
         stop("'player1$", id, "' and 'player2$", id,
              "' must be factors with the same levels")
+    if (!identical(attr(player.one, "contrasts"),
+                   attr(player.two, "contrasts")))
+        stop("'player1$", id, "' and 'player2$", id,
+             "' must have the same contrasts attribute")
     if(is.null(formula)) formula <- reformulate(id)
 
     players <- levels(player.one)
@@ -53,7 +58,8 @@ Diff <- function(player1, player2, formula = NULL, id = "..", data = NULL,
             }
             if (length(sep)) {
                 fixed <- reformulate(c(names(sep), attr(mt, "term.labels"),
-                                       rownames(attr(mt, "factors"))[attr(mt, "offset")]))
+                                       rownames(attr(mt, "factors"))[
+                                           attr(mt, "offset")]))
                 mt <- terms(fixed)
             }
         }
@@ -68,9 +74,16 @@ Diff <- function(player1, player2, formula = NULL, id = "..", data = NULL,
             if (!is.null(refcat)) {
                 mf1[[id]] <- relevel(mf1[[id]], refcat)
                 mf2[[id]] <- relevel(mf2[[id]], refcat)
+                if (!is.null(contrasts)) contrasts[[id]] <- "contr.treatment"
+            } else {
+                ## 'else' defined by contrasts arg/contrasts attr of id factor
+                ## leave refcat NULL
+                if (is.null(contrasts) & 
+                    !is.null(attr(player.one, "contrasts"))){
+                    contrasts <- list()
+                    contrasts[[id]] <- attr(player.one, "contrasts")
+                }
             }
-            else
-                refcat <- levels(mf1[[id]])[1]
         }
         offset <- model.offset(mf1)
         if (!is.null(offset)) offset <- offset - model.offset(mf2)
@@ -94,7 +107,8 @@ Diff <- function(player1, player2, formula = NULL, id = "..", data = NULL,
         ## will need to check for saturation in each set of indexed var
         ## - however as only allowing (1|..) just consider player id for now
 
-        saturated <- qr(na.omit(X))$rank == qr(na.omit(cbind(D, X)))$rank && !idterm
+        saturated <- 
+            qr(na.omit(X))$rank == qr(na.omit(cbind(D, X)))$rank && !idterm
         if (all(X[,1] == 0)) X <- X[, -1, drop = FALSE]
         attr(X, "assign") <- attr(X1, "assign")[-1]
     }
@@ -118,6 +132,7 @@ Diff <- function(player1, player2, formula = NULL, id = "..", data = NULL,
             random <- D[,!sep[[id]], drop = FALSE]
     }
     list(X = X, random = random, offset = offset,
-         term.labels = term.labels, refcat = refcat, saturated = saturated)
+         term.labels = term.labels, refcat = refcat, contrasts = contrasts,
+         saturated = saturated)
 }
 
